/*********************************************************************
 * 
 *    Filename:  catch.c
 *    Author:    Ryan Shillington <shillington@writeme.com>
 *    Description:  This captures key codes and outputs them to a file.
 *  
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 ********************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "catch.h"
#include <string.h>

#define NUM_TRIALS 5
int findBest(char bin[NUM_TRIALS][64]);

int main()
{
  int irport;     //* FILE ID
  int configFile; //* FILE ID
  char configFilename[64];
  int len, i, j;
  char key[64];
  char code[64];
  char bin[NUM_TRIALS][64];
  int largest, bigBin, maxBin;
  char ans = 'y';
  
  //* 1 - Open the irport for transfer
  if ((irport = open("/dev/irport", O_RDWR)) == -1)
  {
    DEBUG(0, "Error opening /dev/irport.\n%s\n", strerror(errno));
    DEBUG(0, "Are you sure that you installed the ir-remote modules?\n\n");
    exit(1);
  }
  
  printf("What would you like to call your device (ie. Sharp, RCA, etc.)?\n");
  scanf("%s", configFilename);

  //* 2 - Delete the configuration file (if there is one), then open a new 
  //* one for writing
  len = strlen(configFilename);
  sprintf(configFilename+len, ".cfg");
  unlink(configFilename);
  if((configFile = open(configFilename, O_RDWR | O_CREAT, 
          S_IRUSR|S_IWUSR | S_IRGRP | S_IROTH)) == -1)
  {
    DEBUG(0, "Error opening/deleting configuration file.\n%s\n", 
        strerror(errno));
    exit(1);
  }
  
  //* 3 - Get as many keys as we want to put in the configuration file
  while (ans == 'y')
  {
    //* 3.1 - Figure out which key to look for
    memset(key, 0, 64);
    printf("What key are you entering: ");
    scanf("%s", key);
    while((strchr(key, '\n') != NULL) && (strchr(key, '\t') != NULL))
    {
      printf("INVALID KEY (Only text please)\nWhat key are you entering: ");
      scanf("%s", key);
    }
    
    //* 3.2 - Get the IR code for the Key
    //* 3.2.1 - Clear the bins
    for (i = 0; i < NUM_TRIALS; i++)
      memset(bin[i], 0, 64);
    
    //* 3.2.2 - Get a number of samples
    printf("\n\nPlace the Remote Control in front of the sensor, and");
    printf("\npress the '%s' button %d times SLOWLY.\n", key, NUM_TRIALS);
    maxBin = 0;
    for(i = 0; i < NUM_TRIALS; i++)
    {
      printf("Trial %d... \n", i+1);
      len = read(irport, code, 62);

      //* 3.2.2.1 Make a bin for the sample
      memcpy(bin[maxBin]+2, code, len);
      bin[maxBin][0] = len;   //* length of the string
      bin[maxBin][1] = 1;     //* count of how many matches
      maxBin++;
      printf("complete.\n");
      for (j = 0; j < len; j++)
      {
        DEBUG(5, "%x ", code[j]);
      }
      DEBUG(4, "\n");
    }
    //* 3.3 - Figure out which codes were the most frequent
    bigBin = findBest(bin);
    
    //* 3.4 - Print out some Debugging information
    DEBUG(5, "Key '%s' is \n", key);
    for (i = 2; i < (bin[bigBin][0] + 2); i++)
    {
      DEBUG(5, "%x ", bin[bigBin][i]);
    }
    DEBUG(5, "\n");
    DEBUG(5, "Length of code is %d\n", len);
    DEBUG(1, "Done.\n");
    
    //* 3.5 - Write the information to the configuration file
    write(configFile, key, strlen(key));
    write(configFile, "\t", 1);
    write(configFile, bin[bigBin]+2, bin[bigBin][0]);
    write(configFile, "\n", 1);
    write(configFile, "\n", 1);
    
    //* 3.6 - Ask about getting a different key
    printf("\n\n***Insert another key (y/n) ?\n");
    ans = getchar();
    while((ans != 'y') && (ans != 'n'))
      ans = getchar();
    getchar();
  }

  //* 4 - Close the files and exit
  close (configFile);
  close(irport);
  return 0;
}


int findBest(char bin[NUM_TRIALS][64])
{
  char single[NUM_TRIALS];
  int count[NUM_TRIALS];
  char curr;
  int accuracy, i, j, k, numMostFreq, mostFreq, maxSingle, len = 0;
    
  accuracy = 0;
  //* Find the shortest trial
  for(i = 0; i < NUM_TRIALS; i++)
    if(len < bin[i][0]) len = bin[i][0];
  
  //* Compare each 8 bits
  for(i = 0; i < len; i++)
  {
    memset(single, 0, NUM_TRIALS);
    memset(count, 0, NUM_TRIALS);
    maxSingle = 0;
    
    //* Find the frequency of all of the different 8 bits
    for(j = 0; j < NUM_TRIALS; j++)
    {
      curr = bin[j][i+2];
      for(k = 0; (k < maxSingle) && (curr != single[k]); k++);
      
      if(k == maxSingle)
      {
        //* Make a new bin
        single[maxSingle] = curr;
        count[maxSingle] = 1;
        maxSingle++;
      } else {
        count[k]++;
      }
    }
    
    //* Find the most frequent
    mostFreq = 0;
    numMostFreq = 0;
    for(j = 0; j < maxSingle; j++)
    {
      if(mostFreq < count[j])
      {
        mostFreq = count[j];
        numMostFreq = j;
      }
    }
    bin[0][i+2] = single[numMostFreq];
    accuracy += count[numMostFreq];
    DEBUG(5, "%d %x %d %d\n", numMostFreq, single[numMostFreq],
        count[numMostFreq], maxSingle);
  }
  bin[0][1] = len;
  DEBUG(1, "Accuracy of reception is %%%3.2f\n", 
      (((float)accuracy) / ((float)(NUM_TRIALS * len))) * 100);
  return 0;
}
